#!/bin/bash
###############################################################################
## vedit - Copyright (c) 2008 Oliver Schneider (assarbad.net)
##
## Little shell script to keep track of the versions of configuration files.
## It is being executed with vedit just like any other editor, however, it
## allows only one file to be given on the command line and checks for this
## condition before continuing.
##
## $Id: vedit.sh,v 1.13 2008/06/03 18:41:41 root Exp $
###############################################################################
[ -n "$TMP" ] || TMP='/tmp'; # Set some temp path
CURRBASENAME=`basename $0`
command pushd `dirname $0` > /dev/null; CURRABSPATH=`pwd`; command popd > /dev/null; # Get the directory in which the script resides
VERSION=`date -r "$CURRABSPATH/$CURRBASENAME" +%Y%m%d.%H%M%S`

###############################################################################
##
## version()
##
## - Takes no parameters
## - Outputs the version information for the script
##
###############################################################################
function version()
{
  echo -e "\033[0;32m${CURRBASENAME%%.*}, version $VERSION\033[0m"
  echo -e "\033[1;30mCopyright (c) ${VERSION:0:4} Oliver Schneider (assarbad.net)\033[0m"
  echo -e ""
}

###############################################################################
##
## cleanup()
##
## - Takes one mandatory parameter, which is the signal code for which the trap
##   was set. Another optional parameter is the EXIT_CODE
## - Will clean up after the script, if the script gets interrupted by whatever
##   means (except for KILL).
##
###############################################################################
function cleanup()
{
  if [ -z "$2" ]; then # Only if we are the actual trap
    EXIT_CODE=$?
    (($1)) && echo -e "\n\n\033[1;33mWARNING: Script has been interrupted ($1), cleaning up!\033[0m"
  else
    EXIT_CODE=$2
  fi
  # Remove the temporary folder, if it has been created
  [ -n "$TEMP_FOLDER" ] && [ -d "$TEMP_FOLDER" ] && { rm -rf "$TEMP_FOLDER"; (($1)) && echo -e "\t\033[1;33mRemoved temporary work folder '$TEMP_FOLDER'\033[0m"; }
  [ -n "$LOCKEDFILE" ] && command ci -q -u "$LOCKEDFILE" && echo -e "\033[0;32mRemoving lock on $LOCKEDFILE\033[0m"
  trap - 0 # Remove trap handler for exit
  exit $EXIT_CODE
}

###############################################################################
##
## set_traps()
##
## - Takes no parameters.
## - This sets the traps for the script (cleanup code).
## - This also creates a unique name for a temporary folder and creates that
##   folder and changes into it with the external PUSHD.
##
###############################################################################
function set_traps()
{
  # Set traps for cleanup
  for signal in 0 1 2 3 6 14 15; do
    trap "cleanup $signal" $signal
  done
  # Create a temporary folder for our use
  until [ -n "$TEMP_FOLDER" -a ! -d "$TEMP_FOLDER" ]; do
    TEMP_FOLDER="$TMP/$(date +%Y%m%d%H%M%S)_${CURRBASENAME%.*}_${RANDOM}${RANDOM}${RANDOM}"
  done
  mkdir -p -m 0700 "$TEMP_FOLDER" || { echo "Could not create the temporary folder '$TEMP_FOLDER'."; exit 1; }
}

# Check the command line
(( $# == 1 )) || { version; echo "Missing command line argument. Give the file to edit!"; exit 1; }
[ \( "$1" == "-v" \) -o \( "$1" == "--version" \) ] && { version; exit 0; }
FNAME="$1"
[ -f "$FNAME" ] || { version; echo "The file $FNAME does not exist or is not an ordinary file."; exit 1; }
# Get the absolute path name
command pushd `dirname $FNAME` > /dev/null; FDIR=`pwd`; command popd > /dev/null;
# Make sure we use the absolute path
FNAME="$FDIR/${FNAME##*/}"
set_traps;
# 1. Take a copy of the file into a temporary folder after checking it in
if [ -f "$FDIR/RCS/${FNAME##*/},v" ]; then
  if [ -f "$FNAME" ]; then
    # Compare the working copy with the one in the RCS and if needed, check it in
    ( rcsdiff "$FNAME" ) > /dev/null 2>&1 || { echo "Changes detected. Checking in those changes first." && rcs -q -l "$FNAME" > /dev/null 2>&1 && command ci -q -m"Automatic checkin of version change outside of vedit" "$FNAME"; }
  fi
  command co -q -l "$FNAME" > /dev/null 2>&1 && LOCKEDFILE="$FNAME" && echo -e "\033[0;32mSet lock on file $LOCKEDFILE\033[0m"
fi
cp "$FNAME" "$TEMP_FOLDER/" || { version; echo "Could not take a working copy of the file."; exit 1; }
# 2. Call the editor with the file as parameter
editor "$TEMP_FOLDER/${FNAME##*/}"
# 3. Compare current version with the working copy and exit without error if they are equal
[ -f "$FDIR/RCS/${FNAME##*/},v" ] && ( diff "$TEMP_FOLDER/${FNAME##*/}" "$FNAME" ) > /dev/null 2>&1 && { echo "Keeping current version of '$FNAME'."; exit 0; }
# 4. If the file is already versioned in the RCS folder it was locked in step 1, so
#    overwrite the original file and exit (pass control the the exit handler).
if [ -n "$LOCKEDFILE" -a -f "$LOCKEDFILE" ]; then
  cat "$TEMP_FOLDER/${FNAME##*/}" > "$LOCKEDFILE"
  # The cleanup handler will deal with unlocking and checkin
  exit 0;
fi
# 5. Create the RCS/ folder if it doesn't exist just yet
[ -d "$FDIR/RCS" ] || mkdir "$FDIR/RCS"
# 6. Check in an initial version of the file and fetch it back immediately
echo -e "\033[0;32mInitial version of file $FNAME\033[0m" && cat "$TEMP_FOLDER/${FNAME##*/}" > "$FNAME" && command ci -i "$FNAME" && command co -q "$FNAME" > /dev/null 2>&1
